@@ -21,12 +21,14 @@
 /* ../appl/bakslv.f
    -- translated by f2c (version of 1 June 1993 23:00:00).
    -- and hand edited by Martin Maechler.
+   -- Modified to use level-3 BLAS, Douglas Bates, May, 2001
    */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#include "Rinternals.h"
 #include "R_ext/Linpack.h"
 #include "R_ext/Applic.h"
 
@@ -43,7 +45,8 @@ void bakslv(double *t, int *ldt, int *n,
 
  * where t is a triangular matrix of order n.
  * The subroutine handles the multiple right-hand side case.
- * It is really just a wrapper for the linpack subroutine dtrsl.
+ * It was just a wrapper for the linpack subroutine dtrsl, now it calls
+ * the level-3 blas subroutine dtrsm.
 
  * on entry
 
@@ -80,7 +83,33 @@ void bakslv(double *t, int *ldt, int *n,
  *	       info contains zero if the system is nonsingular.
  *	       otherwise info contains the index of
  *	       the first zero diagonal element of t.
+ *
+ * subroutines and functions
+ *     blas:    dcopy
+ *     blas3:   dtrsm
+ */
+    char *side = "L", *uplo, *transa, *diag = "N";
+    int i, ione = 1, j, nn = *n;
+    double one = 1.0;
 
+    *info = 0;
+    for(i = 0; i < nn; i++) {	/* check for zeros on diagonal */
+	if (t[i * (*ldt + 1)] == 0.0) {
+	    *info = i + 1;
+	    return;
+	}
+    }
+    for(j = 0; j < *nb; j++) {  /* copy b to x */
+       F77_CALL(dcopy)(n, &b[j * *ldb], &ione, &x[j * *ldb], &ione);
+    }
+    transa = ((*job) / 10) ? "T" : "N";
+    uplo = ((*job) % 10) ? "U" : "L";
+    if (*n > 0 && *nb > 0 && *ldt > 0 && *ldb > 0) {
+	F77_CALL(dtrsm)(side, uplo, transa, diag, n, nb, &one,
+			t, ldt, x, ldb);
+    }
+}
+#ifdef OLD
  * subroutines and functions
 
  *     linpack: dtrsl (t,ldt,n, b,job,info)
@@ -103,3 +132,5 @@ void bakslv(double *t, int *ldt, int *n,
        }
     }
 }
+#endif
+
