@@ -25,19 +25,18 @@
 #include "Startup.h"
 #define PSIGNAL
 #include "psignal.h"
-#include "globalvar.h"
 #include "../getline/getline.h"
 
 extern void cmdlineoptions(int, char **);
 extern void readconsolecfg();
 extern int initapp(int, char **);
 extern void Rf_mainloop(void);
-extern UImode CharacterMode;
-extern int UserBreak;
-extern int R_Interactive;
-extern int R_HistorySize;
-extern int R_RestoreHistory;
-extern char *R_HistoryFile;
+__declspec(dllimport) extern UImode CharacterMode;
+__declspec(dllimport) extern int UserBreak;
+__declspec(dllimport) extern int R_Interactive;
+__declspec(dllimport) extern int R_HistorySize;
+__declspec(dllimport) extern int R_RestoreHistory;
+__declspec(dllimport) extern char *R_HistoryFile;
 
 extern char *getDLLVersion();
 extern void saveConsoleTitle();
