@@ -14,8 +14,9 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
+
 /*
  *
  * file: clipboard.c --
@@ -28,6 +29,7 @@
  */
 
 #include "internal.h"
+#include "rui.h"
 
 
 void copytoclipboard(drawing sb)
@@ -47,7 +49,7 @@ void copytoclipboard(drawing sb)
     DeleteDC(hdcDest);
 
     if (!OpenClipboard(NULL) || !EmptyClipboard()) {
-	askok("Unable to open the clipboard");
+	R_ShowMessage("Unable to open the clipboard");
 	DeleteObject(hbmpNew);
 	return;
     }
@@ -55,3 +57,50 @@ void copytoclipboard(drawing sb)
     CloseClipboard();
     return;
 }
+
+int copystringtoclipboard(char *str)
+{
+    HGLOBAL hglb;
+    char *s;
+    int ll = strlen(str) + 1;
+
+    if (!(hglb = GlobalAlloc(GHND, ll))){
+        R_ShowMessage("Insufficient memory: cell not copied to the clipboard");
+	return 1;
+    }
+    if (!(s = (char *)GlobalLock(hglb))){
+        R_ShowMessage("Insufficient memory: cell not copied to the clipboard");
+	return 1;
+    }
+    strcpy(s, str);
+    GlobalUnlock(hglb);
+    if (!OpenClipboard(NULL) || !EmptyClipboard()) {
+        R_ShowMessage("Unable to open the clipboard");
+        GlobalFree(hglb);
+        return 1;
+    }
+    SetClipboardData(CF_TEXT, hglb);
+    CloseClipboard();
+    return 0;
+}
+
+int getstringfromclipboard(char * str, int n)
+{
+    HGLOBAL hglb;
+    char *pc;
+
+    if ( OpenClipboard(NULL) &&
+         (hglb = GetClipboardData(CF_TEXT)) &&
+         (pc = (char *)GlobalLock(hglb))) {
+	strncpy(str, pc, n);
+	str[n+1] = '\0';
+        GlobalUnlock(hglb);
+	CloseClipboard();
+	return 0;
+    } else return 1;
+}
+
+int clipboardhastext()
+{
+    return (int) IsClipboardFormatAvailable(CF_TEXT);
+}
