@@ -71,12 +71,15 @@ function(formula, data = NULL, ..., subset, na.action)
 {
     if(missing(formula) || (length(formula) != 3))
         stop("formula missing or incorrect")
-    if(missing(na.action))
-        na.action <- getOption("na.action")
+    ## <FIXME>
+    ## Remove `na.action' in 1.4.
+    if(!missing(na.action))
+        warning("argument `na.action' is deprecated")
     m <- match.call(expand.dots = FALSE)
     if(is.matrix(eval(m$data, parent.frame())))
         m$data <- as.data.frame(data)
-    m$... <- NULL
+    m$... <- m$na.action <- NULL
+    ## </FIXME>
     m[[1]] <- as.name("model.frame")
     mf <- eval(m, parent.frame())
     response <- attr(attr(mf, "terms"), "response")
