@@ -85,9 +85,9 @@ old.packages <- function(lib.loc=.lib.loc, CRAN=getOption("CRAN"),
     newerVersion <- function(a, b){
         a <- as.integer(strsplit(a, "[\\.-]")[[1]])
         b <- as.integer(strsplit(b, "[\\.-]")[[1]])
-        if(is.na(a))
+        if(any(is.na(a)))
             return(FALSE)
-        if(is.na(b))
+        if(any(is.na(b)))
             return(TRUE)
         for(k in 1:length(a)){
             if(k <= length(b)){
