@@ -170,7 +170,7 @@ help <-
                         sep="")
                     A <- cbind(package=pkgs, library=libs)
                     rownames(A) <- 1:nrow(A)
-                    print(A, quote=F)
+                    print(A, quote=FALSE)
                 } else {
                     stop(paste("No documentation for `", topic,
                                "' in specified packages and libraries:\n",
