@@ -33,14 +33,16 @@ princomp <- function(x, cor = FALSE, scores = TRUE, covmat = NULL,
     }
     cn <- paste("Comp.", 1:ncol(cv), sep = "")
     names(ev) <- cn
-    dimnames(edc$vectors) <- list(dimnames(x)[[2]], cn)
+    dimnames(edc$vectors) <- if(missing(x))
+        list(dimnames(cv)[[2]], cn) else list(dimnames(x)[[2]], cn)
     sdev <- sqrt(ev)
-    sc <- if (cor) sds else rep(1, ncol(z))
+    sc <- if (cor) sds else rep(1, ncol(cv))
     names(sc) <- colnames(cv)
     scr <- if (scores && !missing(x))
         scale(z, center = TRUE, scale = sc) %*% edc$vectors
     if (is.null(cen)) cen <- rep(NA, nrow(cv))
-    edc <- list(sdev = sdev, loadings = edc$vectors,
+    edc <- list(sdev = sdev,
+                loadings = structure(edc$vectors, class="loadings"),
                 center = cen, scale = sc, n.obs = n.obs,
                 scores = scr, call = match.call())
     ## The Splus function also return list elements factor.sdev,
