@@ -97,10 +97,11 @@ extern pascal	OSErr	FSpGetFullPath(const FSSpec *, short *,  Handle *);
 
 
 extern RGBColor	                   tempTypeColour;
+#define kMaxFlavors 7
 
 OSStatus ReadTextFile ( const FSSpec * pFileSpec, WEReference we )
 {
-	const int		kMaxFlavors = 7 ;
+//	const int			kMaxFlavors = 7 ;
 	SInt16			dataForkRefNum = kFileNotOpened ;
 	SInt16			resForkRefNum = kFileNotOpened ;
 	Handle			hText = nil ;
@@ -382,7 +383,7 @@ cleanup :
 	}
 
     if(pFileSpec)
-     SetWTitle ( FrontWindow(), &pFileSpec->name ) ;
+     SetWTitle ( FrontWindow(), (char const *)&pFileSpec->name ) ;
    
    
 	return err ;
@@ -392,7 +393,7 @@ cleanup :
 
 OSStatus ReadUnicodeTextFile ( const FSSpec * pFileSpec, WEReference we )
 {
-	const int	kMaxFlavors = 4 ;
+//	const int	kMaxFlavors = 4 ;
 	SInt16		dataForkRefNum = kFileNotOpened ;
 	SInt16		resForkRefNum = kFileNotOpened ;
 	Handle		hUnicodeText = nil ;
