@@ -89,7 +89,7 @@ WindowPtr Help_Windows[MAX_NUM_H_WIN + 1];
 AEIdleUPP gAEIdleUPP;
 EventRecord gERecord;
 Boolean gQuit, gInBackground;
-
+extern Boolean OnOpenSource;
 
 #define kResumeMask             1       /* bit of message field for resume vs. suspend */
 
@@ -111,6 +111,11 @@ extern RGBColor	                   gTypeColour;
 
 #define	kCMDEventClass	'DCMD'  /* Event class command for MacZip */
 #define	kCMDEvent    	'DCMD'  /* Event command                  */
+#define CMDLineSize 2048
+
+char *StrCalloc(unsigned short size);
+char *StrFree(char *strPtr);
+
 
 void DoGenKeyDown ( const EventRecord *event, Boolean Console);
 extern pascal	OSErr	FSpGetFullPath (const FSSpec*, short*, Handle*);
@@ -241,7 +246,8 @@ void DoPathSelect ( WindowRef window )
 
 void DoMouseDown ( const EventRecord * event )
 {
-	WindowRef	we;
+	//WindowRef	we;
+	WEReference	we;
 	WindowPtr	window ;
 	SInt16		partCode ;
     SInt32 		selStart, selEnd;
@@ -878,7 +884,7 @@ static pascal OSErr HandleOpenDocument( const AppleEvent *ae,
         error("No Picture Can be load in this version of R\n");
     }
     if (fileInfo.fdType == 'TEXT'){
-        if( strncmp("TRUE",mac_getenv("OnOpenSource"),4) == 0)
+        if( OnOpenSource )
         	SourceFile(&fileSpec);
         else {    
         DoNew( false);
@@ -979,6 +985,40 @@ static pascal OSErr HandleQuitApplication( const AppleEvent *ae,
 }
 
 
+/* HandleDoCommandLine routine :
+   Description :
+   This event will be execute when the applcation exit normally
+ */
+pascal OSErr  HandleDoCommandLine (AppleEvent *theAppleEvent, AppleEvent* reply, long
+														handlerRefCon)
+{
+	OSErr		err = 0;
+	DescType	returnedType;
+	Size		actualSize;
+	char	   *CMDString;
+
+    reply = reply; handlerRefCon = handlerRefCon; 
+    
+    CMDString = StrCalloc(CMDLineSize);
+     
+	if ((err = AEGetParamPtr(theAppleEvent, keyDirectObject, typeChar, &returnedType,
+								CMDString, CMDLineSize, &actualSize)) != noErr)
+		return err;
+
+	/* check for missing parameters   */
+
+    if (actualSize <= CMDLineSize)
+        CMDString[actualSize] = 0;		/* Terminate the C string    */
+    
+    consolecmd(CMDString);
+                
+    CMDString = StrFree(CMDString);
+
+    return 0;
+}
+
+
+
 /* InitializeEvents: modified to let R interact with other processes
                      such as UnZip tools, Browsers, etc.
                      AppleEvents handlers are now installed only if
@@ -1013,6 +1053,13 @@ OSErr InitializeEvents( void )
 	if ( ( err = WEInstallTSMHandlers( ) ) != noErr )
 	    goto cleanup;
 
+   /* install Handler for CFommandline-Application-Event ('do_CMD')   */
+	  if ( (  err = AEInstallEventHandler(kCMDEventClass,kCMDEvent,
+					NewAEEventHandlerUPP( HandleDoCommandLine ), 0, false )) != noErr )
+	    goto cleanup;
+
+
+
 	gAEIdleUPP = NewAEIdleUPP(idleProc);
   
  cleanup:
@@ -1034,3 +1081,67 @@ void R_startBrowser(char *fileName)
     FSMakeFSSpec(0,0,name,&HelpFile);
     OpenSelection(&HelpFile);
 }
+
+/*
+**  Alloc memory and init it
+**
+*/
+char *StrCalloc(unsigned short size)
+{
+char *strPtr = NULL;
+
+strPtr = calloc(size, sizeof(char));
+return strPtr;
+}
+
+
+
+/*
+**  Release only non NULL pointers
+**
+*/
+char *StrFree(char *strPtr)
+{
+
+if (strPtr != NULL)
+    {
+    free(strPtr);
+    }
+
+return NULL;
+}
+
+
+
+
+/*
+**  Alloc memory and init it
+**
+*/
+char *StrCalloc(unsigned short size)
+{
+char *strPtr = NULL;
+
+strPtr = calloc(size, sizeof(char));
+return strPtr;
+}
+
+
+
+/*
+**  Release only non NULL pointers
+**
+*/
+char *StrFree(char *strPtr)
+{
+
+if (strPtr != NULL)
+    {
+    free(strPtr);
+    }
+
+return NULL;
+}
+
+
+
