@@ -38,7 +38,7 @@
 #include <fp.h> /* Jago */
 #include <Quickdraw.h>
 
-#ifdef HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H   
 #include <config.h>
 #endif
 #include "Defn.h"
@@ -63,6 +63,7 @@ extern Boolean WeArePasting;
 
 
 
+/*
 #define	rAppStringsID			128
 
 enum {
@@ -70,19 +71,19 @@ enum {
 	sTranslationLockedErr,
 	sTranslationErr,
 	sOpeningErr,
-	sReadErr,				// 5
+	sReadErr,				
 	sWriteToBusyFileErr,
 	sBusyOpen,
 	sChooseFile,
 	sChooseFolder,
-	sChooseVolume,			// 10
+	sChooseVolume,		
 	sCreateFolder,
 	sChooseObject,
 	sChooseApp
 };
 
 #define kSelectFolderPrefKey	4
-
+*/
 
 OSErr  doRSave(Boolean *haveCancel)
 {
@@ -316,35 +317,26 @@ OSErr  doSaveAsGraCommand(void)
 */
 OSErr  doCopyAppNameResource(WindowPtr windowPtr)
 {
-
     OSType fileType;
     OSErr osError;
     SInt16 fileRefNum, WinIndex;
-
-
     WinIndex = isGraphicWindow(windowPtr);
     fileType = 'PICT';
-
     FSpCreateResFile(&(gGReference[WinIndex].fileFSSpec),'ABFF',fileType,smSystemScript);
-
     osError = ResError();
     if(osError == noErr)
 	fileRefNum = FSpOpenResFile(&gGReference[WinIndex].fileFSSpec,
 				    fsRdWrPerm);
-
     if(fileRefNum > 0)
 	osError = doCopyGraResource('STR ', -16396, gAppResFileRefNum,
 				    fileRefNum);
     else
 	osError = ResError();
-
     if(osError == noErr)
 	CloseResFile(fileRefNum);
-
     osError = ResError();
     return(osError);
 }
-
 /*  doCopyGraResource
 */
 OSErr  doCopyGraResource(ResType resourceType, SInt16 resourceID,
@@ -354,11 +346,8 @@ OSErr  doCopyGraResource(ResType resourceType, SInt16 resourceID,
     Str255	sourceResourceName;
     ResType	ignoredType;
     SInt16	ignoredID;
-
     UseResFile(sourceFileRefNum);
-
     sourceResourceHdl = GetResource(resourceType,resourceID);
-
     if(sourceResourceHdl != NULL) {
 	GetResInfo(sourceResourceHdl, &ignoredID, &ignoredType,
 		   sourceResourceName);
@@ -369,13 +358,9 @@ OSErr  doCopyGraResource(ResType resourceType, SInt16 resourceID,
 	if(ResError() == noErr)
 	    UpdateResFile(destFileRefNum);
     }
-
     ReleaseResource(sourceResourceHdl);
-
     return(ResError());
 }
-
-
 // *****************************************************************************
 // *
 // *	DoSelectDirectory( )
@@ -391,7 +376,6 @@ OSErr DoSelectDirectory( void )
     Handle 				pathName=NULL;
     char 				path[FILENAME_MAX];
 	OSErr               anErr = noErr;
-
 	theErr = NavGetDefaultDialogOptions( &dialogOptions );
 	
 	GetIndString( dialogOptions.message, rAppStringsID, sChooseFolder );
@@ -406,7 +390,6 @@ OSErr DoSelectDirectory( void )
 								nil);
 	
 	DisposeNavEventUPP( eventUPP );
-
 	if ( theReply.validRecord && theErr == noErr)
 	{
 		// grab the target FSSpec from the AEDescList:	
@@ -414,7 +397,6 @@ OSErr DoSelectDirectory( void )
 		AEKeyword 	keyWord;
 		DescType 	typeCode;
 		Size 		actualSize = 0;
-
 		// there is only one selection here we get only the first AEDescList:
 		if (( theErr = AEGetNthPtr( &(theReply.selection), 1, typeFSS, &keyWord, &typeCode, 
 		         &finalFSSpec, sizeof( FSSpec ), &actualSize )) == noErr )		
@@ -435,4 +417,3 @@ OSErr DoSelectDirectory( void )
 		
 	return theErr;
 }
-
