@@ -44,6 +44,8 @@
  *			 and the Encode.. utils.
  *
  *  Also ./printvector.c,  ./printarray.c
+ *
+ *  do_sink.c moved to connections.c as of 1.3.0
  */
 
 #ifdef HAVE_CONFIG_H
@@ -76,27 +78,6 @@ void PrintDefaults(SEXP rho)
     R_print.width = GetOptionWidth(rho);
 }
 
-SEXP do_sink(SEXP call, SEXP op, SEXP args, SEXP rho)
-{
-    int ifile, closeOnExit;
-
-    ifile = asInteger(CAR(args));
-    closeOnExit = asLogical(CADDR(args));
-    if(closeOnExit == NA_LOGICAL)
-	error("invalid value for closeOnExit");
-    switch_stdout(ifile); /* will open new connection if required */
-    if (R_SinkCon >= 3) {
-	if(R_SinkCon_to_close == 1) con_close(R_SinkCon);
-	else if (R_SinkCon_to_close == 2) {
-	    Rconnection con = getConnection(R_SinkCon);
-	    con->close(con);
-	}
-    }
-    R_SinkCon = R_OutputCon = ifile;
-    R_SinkCon_to_close = closeOnExit;
-    return R_NilValue;
-}
-
 SEXP do_invisible(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     switch (length(args)) {
@@ -717,3 +698,10 @@ int F77_NAME(realp0) (char *label, int *nchar, float *data, int *ndata)
     return(0);
 }
 
+/* Fortran-callable error routine for lapack */
+
+void F77_NAME(xerbla)(char *srname, int *info)
+{
+    error("On entry to %6s parameter number %d had an illegal value",
+	  srname, *info);
+}
