@@ -1959,9 +1959,9 @@ int
 Rf_setX11Display(Display *dpy, double gamma_fac, X_COLORTYPE colormodel, 
 		 int maxcube, Rboolean setHandlers)
 {
-    static int alreadyDone = 0;
+/*    static int alreadyDone = 0;
     if(alreadyDone) return(TRUE);
-    alreadyDone = 1;
+    alreadyDone = 1; */
     display = dpy;
 
 #define SETGAMMA
@@ -2056,7 +2056,7 @@ Rf_setX11DeviceData(DevDesc *dd, x11Desc *xd)
     dd->dp.canRotateText = TRUE;
     dd->dp.canResizeText = TRUE;
     dd->dp.canClip = TRUE;
-    dd->dp.canHAdj = FALSE;
+    dd->dp.canHAdj = 0;
 
     dd->dp.ask = FALSE;
 
